<?php
/**
 * @file
 * payanyway menu items.
 *
 */

// варианты налогообложения:
const paw_kassa_VAT0     = '1104';  // НДС 0%
const paw_kassa_VAT10    = '1103';  // НДС 10%
const paw_kassa_VAT18    = '1102';  // НДС 18%
const paw_kassa_VATNOVAT = '1105';  // НДС не облагается
const paw_kassa_VATWR10  = '1107';  // НДС с рассч. ставкой 10%
const paw_kassa_VATWR18  = '1106';  // НДС с рассч. ставкой 18%

// paw_kassa_sale - функция "печати" чека в кассе kassa.payanyway.ru, вызывать её нужно в обработчике PayURL в случае положительного
//                  ответа (SUCCESS)
//
// Параметры:
// $mnt_id - номер расширенного счёта
// $mnt_transaction_id - id заказа
// $mnt_amount - сумма заказа (в формате Moneta.ru, пример: 100.00)
// $mnt_data_integrity_code - код проверки целостности данных
// $client_email - электронный почтовый ящик клиента
// $inventoryPositions = array( array('name' => 'tovar 1', 'price' => 1, 'quantity' => 2, 'vatTag' => paw_kassa_VATNOVAT),
//                              array('name' => 'tovar 2', 'price' => 3, 'quantity' => 1, 'vatTag' => paw_kassa_VATNOVAT) ) - позиции заказа
//     где name - название товара
//         price - цена за единицу товара
//         quantity - количество единиц товара
//         vatTag - вариант налогообложения
//
function paw_kassa_sale($mnt_id, $mnt_transaction_id, $mnt_amount, $mnt_data_integrity_code, $client_email, $inventoryPositions)
{
	$data = array(
		'id' => $mnt_transaction_id,
		'checkoutDateTime' => date(DATE_ATOM),
		'docNum' => $mnt_transaction_id,
		'docType' => 'SALE',
		'email' => $client_email,
		'responseURL' => 'https://kassa.payanyway.ru/index.php?do=callback',
		'inventPositions' => $inventoryPositions,
		'moneyPositions' => array(array('paymentType' => 'CARD', 'sum' => $mnt_amount))
	);
	$data['signature'] = md5($data['id'].$data['checkoutDateTime'].$mnt_data_integrity_code);
	$jsonData = json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
	$operationUrl = 'https://kassa.payanyway.ru/api/api.php?method=sale&accountid='.$mnt_id;
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $operationUrl);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonData);
	curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array(
		'Content-Type: application/json',
		'Content-Length: '.strlen($jsonData)
	));
	$result = curl_exec($ch);
	curl_close($ch);
	return $result;
}


function uc_payanyway_callback() {
    watchdog('uc_payanyway', 'Receiving new order notification for order !order_id.', array('!order_id' => check_plain($_REQUEST['MNT_TRANSACTION_ID'])));

	if(isset($_REQUEST['MNT_ID']) && isset($_REQUEST['MNT_TRANSACTION_ID']) && isset($_REQUEST['MNT_OPERATION_ID'])
	   && isset($_REQUEST['MNT_AMOUNT']) && isset($_REQUEST['MNT_CURRENCY_CODE']) && isset($_REQUEST['MNT_TEST_MODE'])
	   && isset($_REQUEST['MNT_SIGNATURE']) && isset($_REQUEST['cart_id']))
	{
        $mnt_dataintegrity_code = variable_get('uc_payanyway_mnt_dataintegrity_code', '');
		$mnt_kassa = variable_get('uc_payanyway_mnt_kassa', 0);
        $mnt_signature = md5("{$_REQUEST['MNT_ID']}{$_REQUEST['MNT_TRANSACTION_ID']}{$_REQUEST['MNT_OPERATION_ID']}{$_REQUEST['MNT_AMOUNT']}{$_REQUEST['MNT_CURRENCY_CODE']}{$_REQUEST['MNT_TEST_MODE']}{$mnt_dataintegrity_code}");
        if ($_REQUEST['MNT_SIGNATURE'] == $mnt_signature)
        {
            $order = uc_order_load($_REQUEST['MNT_TRANSACTION_ID']);
//			begin of 54fz addon -->
			if ($mnt_kassa == 1)
			{
				$inventoryPositions = array();
				foreach ($order->products as $product)
				{
					$inventoryPositions[] = array(
						'name' => $product->title,
						'price' => $product->price,
						'quantity' => $product->qty,
						'vatTag' => paw_kassa_VATNOVAT
					);
				}
				foreach ($order->line_items as $item)
				{
					if ($item['type'] == 'shipping')
					{
						$inventoryPositions[] = array(
							'name' => 'Доставка (' . $item['title'] . ')',
							'price' => number_format($item['amount'], 2, '.', ''),
							'quantity' => '1',
							'vatTag' => paw_kassa_VATNOVAT
						);
						break;
					}
				}
				paw_kassa_sale($_REQUEST['MNT_ID'], $_REQUEST['MNT_TRANSACTION_ID'], $_REQUEST["MNT_AMOUNT"], $mnt_dataintegrity_code, $order->primary_email, $inventoryPositions);
			}
//			<-- end of 54fz addon

            uc_cart_empty($_REQUEST['cart_id']);

            // Save changes to order without it's completion.
            $order->order_status = 'payment_received';
            uc_order_save($order);
			uc_payment_enter($_REQUEST['MNT_TRANSACTION_ID'], 'payanyway', $_REQUEST['MNT_AMOUNT'], $order->uid, NULL, $description);
            uc_order_comment_save($order->order_id, 0, t('Payment received.'), 'admin');
            watchdog('uc_payanyway', 'Receiving new order notification for order !order_id. Payment completed!', array('!order_id' => check_plain($_REQUEST['MNT_TRANSACTION_ID'])));
            echo 'SUCCESS';
        }
        else {
            watchdog('uc_payanyway', 'Receiving new order notification for order !order_id. Wrong signature!', array('!order_id' => check_plain($_REQUEST['MNT_TRANSACTION_ID'])));
            echo 'FAIL';
        }
    }
    else {
        watchdog('uc_payanyway', 'Receiving new order notification for order !order_id. Bad request!', array('!order_id' => check_plain($_REQUEST['MNT_TRANSACTION_ID'])));
        echo 'FAIL';
    }
}

function uc_payanyway_payment_end($type) {
  if ($type=='success') {
    if (isset($_SESSION['cart_order'])) {
	  uc_cart_empty(uc_cart_get_id());
      drupal_goto('cart/checkout/complete');
    }
  }
  else {
    unset($_SESSION['cart_order']);
    drupal_set_message( t("Your payment has been cancelled."));
    drupal_goto('cart');
  }
  return $output;
}

function uc_payanyway_invoice() {
	$payanyway_server = variable_get('uc_payanyway_server', '');
	$login = variable_get('uc_payanyway_login', '');
	$password = variable_get('uc_payanyway_password', '');

	require_once (dirname(__FILE__).'/MonetaAPI/MonetaWebService.php');
	switch ($payanyway_server) {
		case 'www.payanyway.ru':
			$service = new MonetaWebService("https://www.moneta.ru/services.wsdl", $login, $password);
			break;
		case 'demo.moneta.ru':
			$service = new MonetaWebService("https://demo.moneta.ru/services.wsdl", $login, $password);
			break;
	}

	try
	{
		// получить данные счета
		$request = new MonetaInvoiceRequest();
		$request->payer = $_REQUEST['paymentSystem_accountId'];
		$request->payee = $_REQUEST['MNT_ID'];
		$request->amount = $_REQUEST['MNT_AMOUNT'];
		$request->clientTransaction = $_REQUEST['MNT_TRANSACTION_ID'];

		if ($_REQUEST['payment_method'] == 'postal')
		{
			$operationInfo = new MonetaOperationInfo();
			$a1 = new MonetaKeyValueAttribute();
			$a1->key = 'mailofrussiaindex';
			$a1->value = $_REQUEST['additionalParameters_mailofrussiaSenderIndex'];
			$operationInfo->addAttribute($a1);
			$a2 = new MonetaKeyValueAttribute();
			$a2->key = 'mailofrussiaaddress';
			$a2->value = $_REQUEST['additionalParameters_mailofrussiaSenderAddress'];
			$operationInfo->addAttribute($a2);
			$a3 = new MonetaKeyValueAttribute();
			$a3->key = 'mailofrussianame';
			$a3->value = $_REQUEST['additionalParameters_mailofrussiaSenderName'];
			$operationInfo->addAttribute($a3);
			$request->operationInfo = $operationInfo;
		}
		elseif ($_REQUEST['payment_method'] == 'euroset')
		{
			$operationInfo = new MonetaOperationInfo();
			$a1 = new MonetaKeyValueAttribute();
			$a1->key = 'rapidamphone';
			$a1->value = $_REQUEST['additionalParameters_rapidaPhone'];
			$operationInfo->addAttribute($a1);
			$request->operationInfo = $operationInfo;
		}
		$response = $service->Invoice($request);
		if ($_REQUEST['payment_method'] == 'euroset')
		{
			$response1 = $service->GetOperationDetailsById($response->transaction);
			foreach ($response1->operation->attribute as $attr)
			{
				if ($attr->key == 'rapidatid')
				{
					$transaction_id = $attr->value;
				}
			}
		}
		else
		{
			$transaction_id = $response->transaction;
		}

		$invoice->status = $response->status;
		$invoice->transaction = str_pad($transaction_id, 10, "0", STR_PAD_LEFT);
		$invoice->system = $_REQUEST['payment_method'];
		$invoice->amount = $_REQUEST['MNT_AMOUNT']." ".$_REQUEST['MNT_CURRENCY_CODE'];
		uc_cart_empty($_REQUEST['cart_id']);
	}
	catch (Exception $e)
	{
		$invoice->status = 'FAILED';
		$invoice->error_message = $e->getMessage();
	}
    unset($_SESSION['cart_order']);

	return uc_payanyway_invoice_print($invoice);
}

function uc_payanyway_invoice_print($invoice)
{
	$output = '';
	if ($invoice->status == 'CREATED')
	{
		$output .= "<h3>".t('Invoice was created')."</h3>";
		switch($invoice->system) {
			case 'postal':
				$output .= "<p>".t("Transaction is registered for processing. To complete your payment please <a target='_blank' href='https://www.payanyway.ru/mailofrussiablank.htm?operationId=!transaction'>print post ticket</a> and proceed with payment in any <a target='_blank' href='http://www.russianpost.ru'>Russian Post</a> post office.To view post ticket in PDF format you should have a program <a target='_blank' href='http://get.adobe.com/reader/'>Adobe Acrobat Reader</a> installed.", 
									array('!transaction'=>$invoice->transaction))."</p>";
				break;
			case 'banktransfer':
				$output .= "<p>".t("Transaction is registered for processing. To complete your payment please <a target='_blank' href='https://www.payanyway.ru/wiretransferreceipt.htm?transactionId=!transaction'>print receipt</a> and make payment in bank office.", 
									array('!transaction'=>$invoice->transaction))."</p>";
				break;
			case 'forward':
				$output .= "<p>".t("Contract number for Forward Mobile is:")." {$invoice->transaction}</p>";
				$output .= "<p>".t("Transaction is registered. Please proceed payment with Forward Mobile system using following <b>MONETA.RU</b> account number:")."</p>";
				$output .= "<p>{$invoice->transaction}</p>";
				$output .= "<p>".t("Total amount:")." {$invoice->amount}</p>";
				break;
			case 'ciberpay':
				$output .= "<p>".t("Contract number for CiberPay is:")." {$invoice->transaction}</p>";
				$output .= "<p>".t("Transaction is registered. Please proceed payment with CiberPay system using following <b>MONETA.RU</b> account number:")."</p>";
				$output .= "<p>{$invoice->transaction}</p>";
				$output .= "<p>".t("Total amount:")." {$invoice->amount}</p>";
				break;
			case 'novoplat':
				$output .= "<p>".t("Contract number for NovoPlat is:")." {$invoice->transaction}</p>";
				$output .= "<p>".t("Transaction is registered. Please proceed payment with NovoPlat system using following <b>PayAnyWay</b> account number:")."</p>";
				$output .= "<p>{$invoice->transaction}</p>";
				$output .= "<p>".t("Total amount:")." {$invoice->amount}</p>";
				break;
			case 'platika':
				$output .= "<p>".t("Contract number for PLATiKA is:")." {$invoice->transaction}</p>";
				$output .= "<p>".t("Transaction is registered. Please proceed payment with PLATiKA system using following <b>MONETA.RU</b> account number:")."</p>";
				$output .= "<p>{$invoice->transaction}</p>";
				$output .= "<p>".t("Total amount:")." {$invoice->amount}</p>";
				break;
			case 'mcb':
				$output .= "<p>".t("Contract number for MosCreditBank is:")." {$invoice->transaction}</p>";
				$output .= "<p>".t("Transaction is registered. Please proceed payment with MosCreditBank cash payment terminals using following <b>MONETA.RU</b> account number:")."</p>";
				$output .= "<p>{$invoice->transaction}</p>";
				$output .= "<p>".t("Total amount:")." {$invoice->amount}</p>";
				break;
			case 'elecsnet':
				$output .= "<p>".t("Contract number for Elecsnet is:")." {$invoice->transaction}</p>";
				$output .= "<p>".t("Transaction is registered. Please proceed payment with Elecsnet cash payment terminals using following <b>PayAnyWay</b> account number:")."</p>";
				$output .= "<p>{$invoice->transaction}</p>";
				$output .= "<p>".t("Total amount:")." {$invoice->amount}</p>";
				break;
			case 'comepay':
				$output .= "<p>".t("Contract number for ComePay is:")." {$invoice->transaction}</p>";
				$output .= "<p>".t("Transaction is registered. Please proceed payment with ComePay cash payment terminals using following <b>PayAnyWay</b> account number:")."</p>";
				$output .= "<p>{$invoice->transaction}</p>";
				$output .= "<p>".t("Total amount:")." {$invoice->amount}</p>";
				break;
			case 'gorod':
				$output .= "<p>".t("Contract number for Federal system GOROD is:")." {$invoice->transaction}</p>";
				$output .= "<p>".t("Transaction is registered. Please proceed payment with Federal system GOROD using following <b>PayAnyWay</b> account number:")."</p>";
				$output .= "<p>{$invoice->transaction}</p>";
				$output .= "<p>".t("Total amount:")." {$invoice->amount}</p>";
				break;
			case 'contact':
				$output .= "<p>".t("Contract number for Contact is:")." {$invoice->transaction}</p>";
				$output .= "<p>".t("Transaction is registered. Please proceed payment with Contact payment system using following <b>MONETA.RU</b> account number:")."</p>";
				$output .= "<p>{$invoice->transaction}</p>";
				$output .= "<p>".t("Total amount:")." {$invoice->amount}</p>";
				break;
			case 'euroset':
				$output .= "<p>".t("Contract number for Euroset is:")." {$invoice->transaction}</p>";
				$output .= "<p>".t("Transaction is registered. Please proceed payment with Euroset using following <b>PayAnyWay</b> account number:")."</p>";
				$output .= "<p>{$invoice->transaction}</p>";
				$output .= "<p>".t("Total amount:")." {$invoice->amount}</p>";
				break;
		}
	}
	else
	{
		$output .= "<h3>".t('Error occured during creating invoice')."</h3>";
		$output .= "<p>".$invoice->error_message."</p>";
	}
	return $output;
}
